#include "run_File.h"
#include "f_util.h"
#include "pico/stdlib.h"
#include "hw_config.h"

#include <stdio.h>
#include <stdlib.h> // malloc() free()
#include <string.h>


char pathName[20] = "pic/10000.bmp";
int scanFileNum = 99999;
char *disPath;

static sd_card_t *sd_get_by_name(const char *const name) {
    for (size_t i = 0; i < sd_get_num(); ++i)
        if (0 == strcmp(sd_get_by_num(i)->pcName, name)) return sd_get_by_num(i);
    // DBG_PRINTF("%s: unknown name %s\n", __func__, name);
    return NULL;
}

static FATFS *sd_get_fs_by_name(const char *name) {
    for (size_t i = 0; i < sd_get_num(); ++i)
        if (0 == strcmp(sd_get_by_num(i)->pcName, name)) return &sd_get_by_num(i)->fatfs;
    // DBG_PRINTF("%s: unknown name %s\n", __func__, name);
    return NULL;
}

void run_mount() {
    const char *arg1 = strtok(NULL, " ");
    if (!arg1) arg1 = sd_get_by_num(0)->pcName;
    FATFS *p_fs = sd_get_fs_by_name(arg1);
    if (!p_fs) {
        printf("Unknown logical drive number: \"%s\"\n", arg1);
        return;
    }
    FRESULT fr = f_mount(p_fs, arg1, 1);
    if (FR_OK != fr) {
        printf("f_mount error: %s (%d)\n", FRESULT_str(fr), fr);
        return;
    }
    sd_card_t *pSD = sd_get_by_name(arg1);
    // myASSERT(pSD);
    pSD->mounted = true;
}

void run_unmount() {
    const char *arg1 = strtok(NULL, " ");
    if (!arg1) arg1 = sd_get_by_num(0)->pcName;
    FATFS *p_fs = sd_get_fs_by_name(arg1);
    if (!p_fs) {
        printf("Unknown logical drive number: \"%s\"\n", arg1);
        return;
    }
    FRESULT fr = f_unmount(arg1);
    if (FR_OK != fr) {
        printf("f_unmount error: %s (%d)\n", FRESULT_str(fr), fr);
        return;
    }
    sd_card_t *pSD = sd_get_by_name(arg1);
    // myASSERT(pSD);
    pSD->mounted = false;
}


void sdInitTest(void)
{
    puts("Hello, world!");

    // See FatFs - Generic FAT Filesystem Module, "Application Interface",
    // http://elm-chan.org/fsw/ff/00index_e.html
    sd_card_t *pSD = sd_get_by_num(0);
    FRESULT fr = f_mount(&pSD->fatfs, pSD->pcName, 1);
    if (FR_OK != fr) panic("f_mount error: %s (%d)\n", FRESULT_str(fr), fr);
    FIL fil;
    const char* const filename = "filename.txt";
    fr = f_open(&fil, filename, FA_OPEN_APPEND | FA_WRITE);
    if (FR_OK != fr && FR_EXIST != fr)
        panic("f_open(%s) error: %s (%d)\n", filename, FRESULT_str(fr), fr);
    if (f_printf(&fil, "Hello, world!\n") < 0) {
        printf("f_printf failed\n");
    }
    fr = f_close(&fil);
    if (FR_OK != fr) {
        printf("f_close error: %s (%d)\n", FRESULT_str(fr), fr);
    }
    f_unmount(pSD->pcName);

    puts("Goodbye, world!");
}

char sdTest(void)
{
    sd_card_t *pSD = sd_get_by_num(0);
    FRESULT fr = f_mount(&pSD->fatfs, pSD->pcName, 1);
    if(FR_OK != fr) {
        return 1;
    }
    else {
        f_unmount(pSD->pcName);
        return 0;
    }
}



static void setPathIndex(int index)
{
    FRESULT fr; /* Return value */
    FIL fil;
    UINT br;

    run_mount();

    fr =  f_open(&fil, "index.txt", FA_OPEN_ALWAYS | FA_WRITE);
    if(FR_OK != fr && FR_EXIST != fr) {
        printf("setPathIndex open error\r\n");
        run_unmount();
        return;
    }
    f_printf(&fil, "%d\r\n", index);
    printf("set index is %d\r\n", index);

    f_close(&fil);
    run_unmount();
}

static int getPathIndex(void)
{
    int index = 0;
    char indexs[10];
    FRESULT fr; /* Return value */
    FIL fil;

    run_mount();

    fr =  f_open(&fil, "index.txt", FA_READ);
    if(FR_OK != fr && FR_EXIST != fr) {
        printf("getPathIndex open error\r\n");
        run_unmount();
        return 0;
    }
    f_gets(indexs, 10, &fil);
    sscanf(indexs, "%d", &index);   // char to int
    if(index > scanFileNum) {
        index = 10000;
        printf("get index over scanFileNum\r\n");    
    }
    printf("get index is %d\r\n", index);
    
    f_close(&fil);
    run_unmount();
    
    return index;
}

void setFilePath(void)
{
    int index = 10000;

    if(isFileExist("index.txt")) {
        printf("index.txt is exist\r\n");
        index = getPathIndex();
    }
    else {
        printf("creat and set Index 10000\r\n");    
        setPathIndex(index);
    }

    // const char *pathName = "pic/10000.bmp";
    pathName[8]='0'+index%10; index=index/10;
    pathName[7]='0'+index%10; index=index/10;
    pathName[6]='0'+index%10; index=index/10;
    pathName[5]='0'+index%10; index=index/10;
    pathName[4]='0'+index%10;

    if(!isFileExist(pathName)) {
        index=10000;
        setPathIndex(index);
        pathName[8]='0';
        pathName[7]='0';
        pathName[6]='0';
        pathName[5]='0';
        pathName[4]='1';
    }

    disPath = pathName;
    printf("setFilePath is %s", disPath);
}

void updatePathIndex(void)
{
    int index = 0;

    index = getPathIndex();
    index++;
    if(index > scanFileNum)
        index = 10000;
    setPathIndex(index);
    
    printf("updatePathIndex index is %d\r\n", index);
}

char isFileExist(const char *path)
{
    FRESULT fr; /* Return value */
    FIL fil;

    run_mount();

    fr =  f_open(&fil, path, FA_READ);
    if(FR_OK != fr && FR_EXIST != fr) {
        printf("%s is not exist\r\n", path);
        run_unmount();
        return 0;
    }
    
    f_close(&fil);
    run_unmount();

    return 1;
}

